import unittest
import collections.abc
from iter_ex import *

class TestEx1(unittest.TestCase):
    """ tests point iteration """

    def test_iter(self):
        p = Point(1, 2, 3)
        try:
            it = iter(p)
        except TypeError:
            self.fail("Point is not iterable")

    def test_correct_values(self):
        p = Point(1, 2, 3)
        x, y, z = p
        self.assertEqual(x, 1)
        self.assertEqual(y, 2)
        self.assertEqual(z, 3)

    def test_stop_iteration(self):
        p = Point(1, 2, 3)
        it = iter(p)
        with self.assertRaises(StopIteration):
            for _ in range(4):
                next(it)



class TestEx2(unittest.TestCase):
    """ tests function infinite """

    def test_delivers_generator(self):
        g = infinite(42)
        self.assertTrue(isinstance(g, collections.abc.Generator))

    def test_delivery(self):
        g = infinite(42)
        l = [next(g) for _ in range(10_000)]
        self.assertTrue(all(x == 42 for x in l))


class TestEx3(unittest.TestCase):
    """ tests split function """
    def test_delivers_generator(self):
        g = split([], '')
        self.assertTrue(isinstance(g, collections.abc.Generator))

    def test_empty(self):
        g = split([], '')
        with self.assertRaises(StopIteration):
            next(g)

    def test_equal_list(self):
        g = split([1, 2, 's', 3, 4, 's', 5, 6], 's')
        self.assertEqual(next(g), [1, 2])
        self.assertEqual(next(g), [3, 4])
        self.assertEqual(next(g), [5, 6])

    def test_uneven_list(self):
        g = split([1, 2, 2, 2, 's', 3, 's', 5, 6], 's')
        self.assertEqual(next(g), [1, 2, 2, 2])
        self.assertEqual(next(g), [3])
        self.assertEqual(next(g), [5, 6])


class TestEx4(unittest.TestCase):
    """ tests function cards """
    def test_generator(self):
        self.assertTrue(isinstance(cards(), collections.abc.Generator))

    def test_length(self):
        self.assertTrue(len(list(cards())), 52)

    def test_card_elements(self):
        card_list = list(cards())
        self.assertTrue((2, 'diamonds') in card_list)
        self.assertTrue((4, 'spades') in card_list)
        self.assertTrue((7, 'clubs') in card_list)
        self.assertTrue((10, 'hearts') in card_list)
        self.assertTrue(('J', 'diamonds') in card_list)
        self.assertTrue(('A', 'clubs') in card_list)
    

if __name__ == '__main__':
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestEx1))

    # TODO uncomment the following test if you proceed

    # tests for exercise 2
    # suite.addTest(unittest.makeSuite(TestEx2))

    # tests for exercise 3
    # suite.addTest(unittest.makeSuite(TestEx3))

    # tests for exercise 4
    # suite.addTest(unittest.makeSuite(TestEx4))

    unittest.TextTestRunner(verbosity=3).run(suite)


